#include "graphics.h"

/* returns position of <x>,<y>'th element in a single dimensional array
 * with <num_col> columns 
*/
int pos(int x, int y, int num_col){
	return y * num_col + x;
}

int c_pos(int x, int y, int rad){
	x = x + rad;
	y = y + rad;
	return pos(x, y, 2*rad+1);
}

/* returns pixel <x>,<y> from <image> or NULL if location is outside of the image
*/
unsigned char* getPixel(TargaImage* image, int x, int y){
	if(x < 0 || x >= image->width || y < 0 || y >= image->height) {
		return NULL;
	} else {
		return image->data + pos(x, y, image->width) * 4;
	}
}

void copy_pxl(unsigned char* src, unsigned char* dest){
	for(int i = 0; i < 4; i++){
		dest[i] = src[i];
	}
}


TargaImage* resize(TargaImage* image, int new_width, int new_height, float(*filter)(float x, float r)){	
	//create new image
	unsigned char* horz_data = new unsigned char[new_width * image->height * 4];
	TargaImage* horz_image = new TargaImage(new_width, image->height, horz_data);
	
	unsigned char* vert_data = new unsigned char[new_width * new_height * 4];
	TargaImage* vert_image = new TargaImage(new_width, new_height, vert_data);
	
	//vert only
	//unsigned char* vert_data = new unsigned char[image->width * new_height * 4];
	//TargaImage* vert_image = new TargaImage(image->width, new_height, vert_data);
	
	resample_horz(image, horz_image, filter);
	resample_vert(horz_image, vert_image, filter);
	//downsample_vert(image, vert_image, filter);

	delete horz_image;
	//return horz_image;

	return vert_image;
}

void resample_vert(TargaImage* image, TargaImage* new_image, float(*filter)(float, float)){	
	float delta_y	= (float)image->height/new_image->height;
	float rad;
	if(delta_y > 1){
		rad		= delta_y;
	} else {
		rad		= l;
	}

	float y0		= delta_y/2.0 - 0.5;

	for(int i = 0; i < image->width; i++){
		for(int j = 0; j < new_image->height; j++){
			unsigned char* new_pxl = getPixel(new_image, i, j);
			new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
			//printf("new_pxl(%d,%d) ", i, j);
			
			double norm = 0;
			float y = y0 + j * delta_y;

			for(int k = ceil(y - rad); k < floor(y + rad); k++){
				//printf("y:%lf (%d,%d) ", y, i, k);
				unsigned char* pxl = getPixel(image, i, k);

				if(pxl != NULL){
					float f_val = filter(y - k, rad);
					norm += f_val;
					
					new_pxl[0] += pxl[0] * f_val;
					new_pxl[1] += pxl[1] * f_val;
					new_pxl[2] += pxl[2] * f_val;
				}
			}
			new_pxl[0] /= norm;
			new_pxl[1] /= norm;
			new_pxl[2] /= norm;

			//cout << endl;
		}
	}
	//printf("dy:%lf rad:%lf y0:%lf ow:%d oh:%d nw:%d nh:%d\n", delta_y, rad, y0, image->width, image->height, new_image->width, new_image->height);
}


void resample_horz(TargaImage* image, TargaImage* new_image, float(*filter)(float, float)){	
	float delta_x	= (float)image->width/new_image->width;
	float rad;
	if(delta_x > 1){
		rad		= delta_x;
	} else {
		rad		= l;
	}

	float x0		= delta_x/2.0 - 0.5;

	for(int j = 0; j < image->height; j++){
		for(int i = 0; i < new_image->width; i++){
			unsigned char* new_pxl = getPixel(new_image, i, j);
			new_pxl[0] = new_pxl[1] = new_pxl[2] = new_pxl[3] = 0;
			//printf("new_pxl(%d,%d) ", i, j);
			
			double norm = 0;
			float x = x0 + i * delta_x;

			for(int k = ceil(x - rad); k < floor(x + rad); k++){
				//printf("y:%lf (%d,%d) ", y, i, k);
				unsigned char* pxl = getPixel(image, k, j);

				if(pxl != NULL){
					float f_val = filter(x - k, rad);
					norm += f_val;
					
					new_pxl[0] += pxl[0] * f_val;
					new_pxl[1] += pxl[1] * f_val;
					new_pxl[2] += pxl[2] * f_val;
				}
			}
			new_pxl[0] /= norm;
			new_pxl[1] /= norm;
			new_pxl[2] /= norm;

			//cout << endl;
		}
	}
	//printf("dy:%lf rad:%lf y0:%lf ow:%d oh:%d nw:%d nh:%d\n", delta_y, rad, y0, image->width, image->height, new_image->width, new_image->height);
}

void FS_dither_pxl(TargaImage* image, int x, int y, float* d_matrix, int d_size){
	float error;
	unsigned char* err_pxl;
	unsigned char* pxl = getPixel(image, x, y);
	
	//if(x == 1) cout << "(" << x << ", " << y << ") " << (int)(pxl[0]);

	//thresh-hold the current pixel
	if(pxl[0] < 128){
		error = (float)pxl[0];
		pxl[0] = 0;
		pxl[1] = 0;
		pxl[2] = 0;

	} else {
		error = (float)pxl[0] - 255;
		pxl[0] = 255;
		pxl[1] = 255;
		pxl[2] = 255;
	}

	//distribute the error
	for(int i = 0; i < d_size; i++){
		for(int j = 0; j < d_size; j++){
			if((err_pxl = getPixel(image, x + (i - d_size/2), y + (j - d_size/2))) != NULL){
				float dis_err = d_matrix[pos(i, j, d_size)] * error;
			
				if(err_pxl[0] + dis_err > 255) {
					err_pxl[0] = 255;
				} else if(err_pxl[0] + dis_err < 0) {
					err_pxl[0] = 0;
				} else {
					err_pxl[0] += (unsigned char)dis_err;
				}
			}
		}
	}
}

//returns a black and white version of the <image>
TargaImage* bw(TargaImage* image){
	float d_right[] = {	0,		0,		0,
						0,		0,	.4375,
					.1875,	.3125,	.0625};

	float d_left[] = {	0,		0,		0,
					.4375,		0,		0,
					.0625,	.3125,	.1875};
	
	//get new grey scaled version of image
	TargaImage* bw_image = greyscale(image);

	//run Floyd-Steinburg Algorithm
	for(int y=0; y < image->height; y++) {		
		if(y%2 == 0){
			for(int x=0; x < image->width; x++){
				FS_dither_pxl(bw_image, x, y, d_right, 3);
			}
		} else {
			for(int x=image->width-1; x >= 0; x--){
				FS_dither_pxl(bw_image, x, y, d_left, 3);
			}
		}
	}

	return bw_image;
}

//returns a greyscaled version of the <image>
TargaImage* greyscale(TargaImage* image){
	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* grey_image = new TargaImage(image->width, image->height, new_data);
	
	//greyscale the image
	for(int x=0; x < image->width; x++){
		for(int y=0; y < image->height; y++){
			unsigned char* pxl  = getPixel(image, x,y);
			unsigned char* pxl_grey = getPixel(grey_image, x, y);
				int L = .3 * pxl[0] + .59 * pxl[1] + .11 * pxl[2];	//Y = 0.30R + 0.59G + 0.11B
				pxl_grey[0]	= L;		// desaturate red
				pxl_grey[1]	= L;		// desaturate green
				pxl_grey[2]	= L;		// desaturate blue
				pxl_grey[3]	= pxl[3];	// leave alpha alone
		}
	}
	return grey_image;
}

void impress_pxl(TargaImage* image, TargaImage* new_image, int x, int y, float* brush, int brush_rad){
	unsigned char* pxl  = getPixel(image, x, y);
	for(int i = -brush_rad; i < brush_rad; i++){
		for(int j = -brush_rad; j < brush_rad; j++){
			//cout << "(x,y)" << x << " " << y << "rad " << i << " " << j << " " << c_pos(i, j, brush_rad) << endl;
			unsigned char* new_pxl  = getPixel(new_image, x+i, y+j);
			if(new_pxl != NULL){
				new_pxl[0] = brush[c_pos(i, j, brush_rad)] * pxl[0] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[0];
				new_pxl[1] = brush[c_pos(i, j, brush_rad)] * pxl[1] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[1];
				new_pxl[2] = brush[c_pos(i, j, brush_rad)] * pxl[2] + (1 - brush[c_pos(i, j, brush_rad)]) * new_pxl[2];
			}
		}
	}
}

TargaImage* impress(TargaImage* image, int brush){
	const int SC_BRUSH = 0;
	const int C_BRUSH  = 1;
	const int SP_BRUSH = 2;
	const int BS_BRUSH = 3;
	const int TL_BRUSH = 4;
	const int ALL_BRUSH = 5;
	int all_prob = 30;

	//small circle brush
	int sc_prob = 10;
	int sc_brush_rad = 3;
	float sc_brush[] =
					{	0,	0,	1,	1,	1,	0,	0,
						0,	1,	1,	1,	1,	1,	0,
						1,	1,	1,	1,	1,	1,	1,
						1,	1,	1,	1,	1,	1,	1,
						1,	1,	1,	1,	1,	1,	1,
						0,	1,	1,	1,	1,	1,	0,
						0,	0,	1,	1,	1,	0,	0};
	
	//cresent brush
	int c_prob = 6;
	int c_brush_rad = 3;
	float c_brush[] =
					{	0,	1,	1,	.3,	.2,	.1,	0,
						0,	1,	1,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						1,	1,	0,	0,	0,	0,	0,
						0,	1,	0,	0,	0,	0,	0,
						0,	1,	1,	1,	1,	.4,	0};
	//spike brush
	//cresent brush
	int sp_prob = 4;
	int sp_brush_rad = 3;
	float sp_brush[] =
					{	0,	0,	0,	0,	0,	1,	0,
						0,	0,	0,	0,	1,	0,	0,
						0,	0,	1,	1,	1,	0,	0,
						0,	1,	1,	1,	0,	0,	0,
						1,	1,	1,	1,	0,	0,	0,
						1,	1,	1,	0,	0,	0,	0,
						1,	1,	1,	0,	0,	0,	0};

	//big square brush
	int bs_prob = 500;
	int bs_brush_rad = 20;
	float *bs_brush = new float[1681];
	for(int i = 0; i < 1681; i++){
		bs_brush[i] = 1;
	}

	//three lines brush
	int tl_prob = 14;
	int tl_brush_rad = 7;
	float tl_brush[] =
					{	.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6, .6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,	
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1,
						.2,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	.2,	
						.1,	.2,	.3,	.4,	.4,	.5,	.6,	.6,	.6,	.5,	.4,	.4,	.3,	.2,	.1};

	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* new_image = new TargaImage(image->width, image->height, new_data);

	
	if(brush != ALL_BRUSH){
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				switch(brush){
					case SC_BRUSH:
						if(rand()%sc_prob == 0)	impress_pxl(image, new_image, x, y, sc_brush, sc_brush_rad);
						break;
					case C_BRUSH:
						if(rand()%c_prob == 0)	impress_pxl(image, new_image, x, y, c_brush, c_brush_rad);
						break;
					case SP_BRUSH:
						if(rand()%sp_prob == 0)	impress_pxl(image, new_image, x, y, sp_brush, sp_brush_rad);
						break;
					case BS_BRUSH:
						if(rand()%bs_prob == 0)	impress_pxl(image, new_image, x, y, bs_brush, bs_brush_rad);
						break;
					case TL_BRUSH:
						if(rand()%tl_prob == 0)	impress_pxl(image, new_image, x, y, tl_brush, tl_brush_rad);
						break;
					default:
						cout << "IMPRESSIONIST usage: p1 <input_image> <output_image> 2 <brush_number(0-5)>" << endl;
						exit(1);
				}
			}
		}
	} else {
		//big square
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(bs_prob*1000)) impress_pxl(image, new_image, x, y, bs_brush, bs_brush_rad);
			}
		}
		//small circle
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(sc_prob*20) == 0) impress_pxl(image, new_image, x, y, sc_brush, sc_brush_rad);
			}
		}
		//cresent
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(c_prob*20) == 0) impress_pxl(image, new_image, x, y, c_brush, c_brush_rad);
			}
		}
		//spike
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(sp_prob*20) == 0) impress_pxl(image, new_image, x, y, sp_brush, sp_brush_rad);
			}
		}
		//threee lines
		for(int y=0; y < new_image->height; y++){
			for(int x=0; x < new_image->width; x++){
				if(rand()%(tl_prob*20) == 0) impress_pxl(image, new_image, x, y, tl_brush, tl_brush_rad);
			}
		}
	}

	return new_image;
}

double normalize(int x, int max){
	float MD2 = max/2.0;
	return x/MD2 - 1;
}

int denormalize(double x, int max){
	float MD2 = max/2.0;
	return (1 + x) * MD2;
}

void warp0_pxl(TargaImage* image, int* x, int* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	
	*x = denormalize(asin(x_norm)/PID2, x_max);
	*y = denormalize(asin(y_norm)/PID2, y_max);
}

void warp1_pxl(TargaImage* image, int* x, int* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	r = sqrt(r);

	*x = denormalize(r * cos(phi), x_max);
	*y = denormalize(r * sin(phi), y_max);
}

void warp2_pxl(TargaImage* image, int* x, int* y){
	int x_max = image->width - 1;
	int y_max = image->height - 1;
	double x_norm = normalize(*x, x_max);
	double y_norm = normalize(*y, y_max);
	double r = sqrt(x_norm*x_norm + y_norm*y_norm);
	double phi = atan2(y_norm, x_norm);
	r = 2*asin(r)/PI;

	*x = denormalize(r * cos(phi), x_max);
	*y = denormalize(r * sin(phi), y_max);
}

void convolve2d_pxl(TargaImage* image, unsigned char* pxl, int x, int y, float(*filter)(float, float), int x_rad, int y_rad){
	pxl[0] = pxl[1] = pxl[2] = pxl[3] = 0
	for(int i = -x_rad; i <= x_rad; i++){
		for(int j = -y_rad; j <= y_rad; j++){
			unsigned char* pxl = getPixel(image, k, j);

			if(pxl != NULL){
				float f_val = filter(x - k, rad);
				norm += f_val;
				
				new_pxl[0] += pxl[0] * f_val;
				new_pxl[1] += pxl[1] * f_val;
				new_pxl[2] += pxl[2] * f_val;
			}
		}
	}
}

TargaImage* warp(TargaImage* image, void(*transform)(TargaImage*, int*, int*), int filter){
	//create new image
	unsigned char* new_data = new unsigned char[image->width * image->height * 4];
	TargaImage* new_image = new TargaImage(image->width, image->height, new_data);

	for(int y=0; y < new_image->height; y++){
		for(int x=0; x < new_image->width; x++){
			int u = x, v = y;
			transform(image, &u, &v);

			if(filter >= 1){
				if(filter == 1){
					for
				int pu = x - 1, pv = y - 1;
				tranform(image, &pu, &pv);
			};



			unsigned char* pxl = getPixel(image, u, v);
			unsigned char* new_pxl = getPixel(new_image, x, y);

			if(pxl != NULL){
				copy_pxl(pxl, new_pxl);
			} else {
				new_pxl[0] = 0;
				new_pxl[1] = 0;
				new_pxl[2] = 0;
			}
		}
	}

	return new_image;
}


