/*	Justin Fiedler
	main.cpp

*/

#include <iostream>
using namespace std;
#include "libtarga.h"
#include "TargaImage.h"
#include "filters.h"
#include "graphics.h"

TargaImage* img_mod;	//modified image
TargaImage* img_in;		//image from user input
char* img_out;			//name of file to save image to
int opt;				//option selected by user
bool s_flag = true;		//flag to control saving the file

const int RESIZE		= 0;
const int BLACK_WHITE	= 1;
const int IMPRESSION	= 2;
const int WARP			= 3;

const int BOX_FILTER		= 0;
const int TENT_FILTER		= 1;
const int BSPLINE_FILTER	= 2;

int main(int argc, char** args)
{	if(argc >= 4){
		//get input values
		// p1 <input_image> <output_image> <opt>
		img_in = TargaImage::Load_Image(args[1]);
		img_out = args[2];
		if(img_in == NULL){
			cerr << "Failed to load input image " << args[1] << endl;
			return 1;
		}
		opt = atoi(args[3]);
		//choose an option
		switch(opt) {
			case RESIZE:
				//p1 <input_image> <output_image> 0 <new_pxl_width> <new_pxl_height>
				if(argc == 7){
					cout << "Resizing Image...";
					int new_width = atoi(args[4]);
					int new_height = atoi(args[5]);
					int filter = atoi(args[6]);
					
					switch(filter){
						case BOX_FILTER:
							img_mod = resize(img_in, new_width, new_height, &box_filter);
							break;
						case TENT_FILTER:
							img_mod = resize(img_in, new_width, new_height, &tentr_filter);
							break;
						case BSPLINE_FILTER:
							img_mod = resize(img_in, new_width, new_height, &bspline_filter);
							break;
						cerr << "RESIZE usage: p1 <input_image> <output_image> 0 <new_pxl_width> <new_pxl_height> <filter_number(0-2)>" << endl;
						return 1;
					}

					cout << "complete" << endl;
				} else {
					cout << "RESIZE usage: p1 <input_image> <output_image> 0 <new_pxl_width> <new_pxl_height> <filter_number>" << endl;
					return 1;
				}
				break;
			case BLACK_WHITE:
				//p1 <input_image> <output_image> 1
				if(argc == 4){
					cout << "Black and Whiting Image...";
					img_mod = bw(img_in);
					cout << "complete" << endl;
				} else {
					cout << "BLACK AND WHITE usage: p1 <input_image> <output_image> 1" << endl;
					return 1;
				}
				break;
			case IMPRESSION:
				//p1 <input_image> <output_image> 2
				if(argc == 5){
					int brush = atoi(args[4]);
					cout << "Impressionisting Image...";
					img_mod = impress(img_in, brush);
					cout << "complete" << endl;
				} else {
					cout << "IMPRESSIONIST usage: p1 <input_image> <output_image> 2 <brush_number(0-5)>" << endl;
					return 1;
				}
				break;
			case WARP:
				//p1 <input_image> <output_image> 3 <warp_number>
				if(argc == 6){
					int w = atoi(args[4]);
					int filter = atoi(args[5]);
					cout << "Warping Image...";
					if(w == 0){
						img_mod = warp(img_in, &warp0_pxl, filter);
					} else if(w == 1) {
						img_mod = warp(img_in, &warp1_pxl, filter);
					} else if(w == 2) {
						img_mod = warp(img_in, &warp2_pxl, filter);
					} else {
						cout << "WARP usage: p1 <input_image> <output_image> 3 <warp_number(0-2)> <filter_number(0-3)>" << endl;
						return 1;
					}
					cout << "complete" << endl;
				} else {
					cout << "WARP usage: p1 <input_image> <output_image> 3 <warp_number(0-2)> <filter_number(0-3)>" << endl;
					return 1;
				}
				break;
			default:
				cerr << "Please enter a valid option(0-3)" << endl;
				return 1;
		}	
		//save image
		if(!img_mod->Save_Image(img_out)){
			cerr << "Image failed to save to " << img_out << endl;
		}
		delete img_mod;
		
		/* run all at once (sometimes i'm too lazy for command line)
		cout << "Reducing Image..."; img_mod = reduce(img_in, new_width, new_height); cout << "complete" << endl; 
		if(!img_mod->Save_Image("reduce.tga")) cout << "save failed" << endl;
		cout << "Enlarging Image..."; img_mod = enlarge(img_in, new_width, new_height); cout << "complete" << endl;
		if(!img_mod->Save_Image("enlarge.tga")) cout << "save failed" << endl;
		cout << "Black and Whiting Image..."; img_mod = bw(img_in); cout << "complete" << endl;
		if(!img_mod->Save_Image("bw.tga")) cout << "save failed" << endl;
		cout << "Impressionisting Imaging..."; img_mod = impress(img_in); cout << "complete" << endl;
		if(!img_mod->Save_Image("impress.tga")) cout << "save failed" << endl;
		*/
		
	} else { //recite error in case you befuddle the command line
		cerr << "usage: p1 <input_image> <output_image> <option> <flags>";
	}

	
}